#include<string.h>
#include<stdlib.h>

#include"GL_helper.h"
#include"zpr.h"

#include<unistd.h>
#include<time.h>

#include<GL/glut.h>

void handleIdle(void);
void handleMenu(int ID);

/******************************************************************************
 Initializes GLUT.
******************************************************************************/
void initGlut(int argc, char* argv[])
{
    // Initialize GLUT.
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize(500, 500);
    glutCreateWindow("ArgonGL");

    // Setup GLUT callbacks.
    glutIdleFunc(handleIdle);

    // Setup GLUT popup menu.
    glutCreateMenu(handleMenu); 
    glutAddMenuEntry("Quit", 0);
    glutAttachMenu(GLUT_RIGHT_BUTTON);

	initGraphics();
	zprInit();
}

/******************************************************************************    
 Uses the haptic device coordinate space as model space for graphics.
 Defines orthographic projection to fit it.
 LLB: Low, Left, Back point of device workspace.
 TRF: Top, Right, Front point of device workspace. 
******************************************************************************/
void initGraphics()//const hduVector3Dd &LLB, const hduVector3Dd &TRF)
{
    // Setup perspective projection.
    glMatrixMode(GL_PROJECTION); 
    glLoadIdentity();

    double centerScreen[3];
    centerScreen[0] = 0; //(-210.0 +  210.0)/2.0;
    centerScreen[1] = 0; //(-110.0 +  205.0)/2.0;
    centerScreen[2] = 0; //( -85.0 +  130.0)/2.0;

    double screenDims[3];
    screenDims[0] = 5; //(210.0 +  210.0);
    screenDims[1] = 5; //(110.0 +  205.0);
    screenDims[2] = 5; //( 85.0 +  130.0);

    double maxDimXY = screenDims[0] > screenDims[1] ? 
        screenDims[0] : screenDims[1];
    double maxDim = maxDimXY > screenDims[2] ? 
        maxDimXY : screenDims[2];
    maxDim /= 2.0;

    glOrtho(centerScreen[0]-maxDim, centerScreen[0]+maxDim, 
            centerScreen[1]-maxDim, centerScreen[1]+maxDim,
            centerScreen[2]-maxDim, centerScreen[2]+maxDim);
    
    glShadeModel(GL_SMOOTH);

    // Setup model transformations.
    glMatrixMode(GL_MODELVIEW); 
    glLoadIdentity();
    glDisable(GL_DEPTH_TEST);
}


/******************************************************************************    
 Sets up graphics pipeline, lights etc.
******************************************************************************/
void setupGraphicsState()
{
    glShadeModel(GL_SMOOTH);

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);
    glEnable(GL_LIGHT_MODEL_TWO_SIDE);
    glShadeModel(GL_SMOOTH);
    
    GLfloat lightZeroPosition[] = { 10.0, 4.0, 10.0, 0.0 };
    GLfloat lightZeroColor[] = { 0.6, 0.6, 0.6, 1.0 }; // green-tinted.
    GLfloat lightOnePosition[] = { -1.0, -2.0, -10.0, 0.0 };
    GLfloat lightOneColor[] = { 0.6, 0.6, 0.6, 1.0 }; // red-tinted.
    
    glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, 1);
    glLightfv(GL_LIGHT0, GL_POSITION, lightZeroPosition);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightZeroColor);
    glLightfv(GL_LIGHT1, GL_POSITION, lightOnePosition);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOneColor);
    glEnable(GL_LIGHT0);
    glEnable(GL_LIGHT1);
}


/******************************************************************************
 Draws a sphere to represent an electric charge.
******************************************************************************/
void drawSphere(double pos[4], float color[4], double sphereRadius)
{
	static GLuint gSphereDisplayList = 0;
	static double radius=0;

    glMatrixMode(GL_MODELVIEW); 
    glPushMatrix();
    if (!gSphereDisplayList || sphereRadius!=radius) {

		if(gSphereDisplayList) {
			glDeleteLists(gSphereDisplayList, 1);
		}

		radius=sphereRadius;
		GLUquadricObj *qobj = 0;
        gSphereDisplayList = glGenLists(1);
        glNewList(gSphereDisplayList, GL_COMPILE);
        qobj = gluNewQuadric();
               
		gluSphere(qobj, sphereRadius, 20, 20); 
    
        gluDeleteQuadric(qobj);
        glEndList();
    }
    glEnable(GL_LIGHTING);
    glTranslatef(pos[0], pos[1], pos[2]);
    glEnable(GL_COLOR_MATERIAL);
    glColor4fv(color);
    glCallList(gSphereDisplayList);
    glPopMatrix();
}


/*******************************************************************************
  Called periodically by the GLUT framework.
*******************************************************************************/
void handleIdle(void)
{
    glutPostRedisplay();

}

/******************************************************************************
 Popup menu handler
******************************************************************************/
void handleMenu(int ID)
{
    switch(ID) 
    {
        case 0:
            exit(0);
            break;
    }
}



