#ifndef __ARGON_H__
#define __ARGON_H__


#include<math.h>
#include<stdio.h>

typedef struct {
	int n;
	int N;
	double m;
	double e;
	double R;
	double f;
	double L;

	double a;
	double T_0;

	double tau;
	int S_o;
	int S_d;

	int S_out;
	int S_avs;
} t_parameters;

typedef struct {
	double x;
	double y;
	double z;
	double crap;
} t_coord;

typedef struct {
	t_coord *r;
	t_coord *p;
	t_coord *F;
	int N;

	int *locked;

	double *damping;
	
	double V;
	double P;
	double H;
	double T;

	double t;

	int step;
} t_state;

#define K_BOLTZMANN 8.31e-3

t_state *alloc_state(int N);
void free_state(t_state *state);
void copy_state(t_state *dest, t_state *src);

void load_parameters(t_parameters *pars, char *fname);
void save_state(FILE *f, t_state *state);
void save_state_xyz(FILE *f, t_state *state);

void init_state(t_state *state, t_parameters *pars);
void compute_forces(t_state *state, t_parameters *pars);
void one_step(t_state *state, t_parameters *pars);

double dist(t_coord r1, t_coord r2);
t_coord scale(t_coord r, double s);

#endif
